#
#*===================================================================
#*
#* Licensed Materials - Property of IBM
#*
#* CA12
#*
#* (c) Copyright IBM Corp. 2014  All Rights Reserved
#*
#* US Government Users Restricted Rights - Use, duplication or
#* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#*
#*===================================================================
#
import subprocess
import maestro
import logging
import urllib2
import base64
from xml.dom import minidom 
import glob
import os

logger = logging.getLogger("CICSAPP/install.py")

logger.debug("Start of CICSAPP install.py")

xparms = maestro.xparms
node = maestro.node

scriptdir = node['scriptdir']
utilpath = scriptdir + '/WAS/CICSAPP'

cicsInstallDir = node['parts']['CICSAPP']['installDir']

bindingsUrl = xparms['appbundle']
bindingsFile = cicsInstallDir+'/'+bindingsUrl.rsplit('/')[-1]
logger.debug('Bindings File [' + bindingsFile + ']')
#Download the bindings zip and unpack (maestro.downloadx)
maestro.downloadx(bindingsUrl, scriptdir)

cmciUrl = node['parts']['CICSAPP']['platformHost']
cmciPort = str(node['parts']['CICSAPP']['platformPort'])
cicsPlex = node['parts']['CICSAPP']['cicsPlex']
userId = node['parts']['CICSAPP']['userId']
password = maestro.decode(node['parts']['CICSAPP']['password'])

logger.debug('CMCI connection to ' + cmciUrl + ':' + cmciPort)

maestro.firewall.open_tcpout(dest=cmciUrl, dport=cmciPort)
base64string = base64.encodestring('%s:%s' % (userId, password))[:-1]
authheader = 'Basic %s' % base64string

try:
    #Open an HTTP connection to CMCI to get the installed CICSplex entries
    request = urllib2.Request('http://'+cmciUrl+':'+cmciPort+'/CICSSystemManagement/CICSCICSplex')
    request.add_header('Authorization', authheader)
    u = urllib2.urlopen(request)
    data = u.read();
    logger.debug("CICSplex data response from CICS: " + data)
    
    #check that CICSplex is available
    plexAvailable = False
    doc = minidom.parseString(data)
    cicsplexList = doc.getElementsByTagName("cicscicsplex")
    for cicsplexInstance in cicsplexList:
        plexName = str(cicsplexInstance.getAttribute("plexname"))
        logger.debug("Found CICSplex " + plexName)
        if plexName == cicsPlex:
            plexAvailable = True
            break
        #endif
    #endfor
    if not plexAvailable:
        logger.error("CICSplex " + cicsPlex + " is not available")
        maestro.role_status = "ERROR"
        raise Exception("Configured CICSplex is not available")
    #endif
    logger.debug("Configured CICSplex is available")
except urllib2.URLError as e:
    logger.error("Unable to connect to CMCI on " + cmciUrl + ":" + cmciPort + ". Reason: " + str(e))
    maestro.role_status = "ERROR"
    raise e

#Established a CMCI connection is possible and CICSplex exists. Grab some info from binding zip
logger.debug('Extracting application information from binding file: ' + scriptdir + '/*/META-INF/appbinding.xml')
list = glob.glob(scriptdir + '/*/META-INF/appbinding.xml')
bindingdoc = minidom.parse(list[0])
platform = str(bindingdoc.getElementsByTagName('appbinding:platform')[0].getAttribute('name'))
applicationElement = bindingdoc.getElementsByTagName('appbinding:application')[0]
application = str(applicationElement.getAttribute('name'))
applicationVer = str(applicationElement.getAttribute('majorVersion')) + '.' + str(applicationElement.getAttribute('minorVersion')) + '.' + str(applicationElement.getAttribute('microVersion'))
bindingElement = bindingdoc.getElementsByTagName('appbinding:appbinding')[0]
binding = str(bindingElement.getAttribute('name'))
bindingVer = str(bindingElement.getAttribute('majorVersion')) + '.' + str(bindingElement.getAttribute('minorVersion')) + '.' + str(bindingElement.getAttribute('microVersion'))
platformDef = ''
platformHome = ''

logger.debug("Binding file contains names Platform (" + platform + ") Application (" + application + ") Binding (" + binding + ").")
if binding == '' or platform == '' or application == '':
    logger.error("Platform, application or binding name is empty in the binding file.")
    maestro.role_status = "ERROR"
    raise Exception("Platform, application or binding name is empty in the binding file.")
#endif

#Check the CICS platform is installed
try:
    request = urllib2.Request('http://'+cmciUrl+':'+cmciPort+'/CICSSystemManagement/CICSPlatform/'+cicsPlex)
    request.add_header('Authorization', authheader)
    u = urllib2.urlopen(request)
    platformData = u.read()
    logger.debug("Platform data response from CICS: " + platformData)
    
    platformDoc = minidom.parseString(platformData)
    platformList = platformDoc.getElementsByTagName('cicsplatform')
    platformFound = False
    for platformResource in platformList:
        platformName = str(platformResource.getAttribute('platform'))
        logger.debug("Found platform: " + platformName)
        if platformName == platform:
            #Found the platform, now check the status which could be ACTIVE, INACTIVE or PARTIAL
            #The application could still be ENABLED in a PARTIAL platform.
            #I guess it's our choice whether we continue to check application or error due to state of platform
            platformStatus = str(platformResource.getAttribute('status'))
            logger.debug("Platform status is: " + platformStatus)
            if platformStatus == 'INACTIVE':
                logger.error('Platform ' + platformName + ' is INACTIVE in CICSplex ' + cicsPlex)
                maestro.role_status = "ERROR"
                raise Exception('Platform ' + platformName + ' is INACTIVE in CICSplex ' + cicsPlex)
            #endif
            platformFound = True
            platformDef = str(platformResource.getAttribute('platdef'))
            platformHome = str(platformResource.getAttribute('plathome'))
            break
        #endif
    #endfor
    if not platformFound:
        logger.error('Platform ' + platform + ' not found in CICSplex ' + cicsPlex)
        maestro.role_status = "ERROR"
        raise Exception('Platform ' + platform + ' not found in CICSplex ' + cicsPlex)
    #endif
except urllib2.URLError as e:
    logger.error("Unable to receive CICS platform data via CMCI http://" + cmciUrl + ":" + cmciPort + "/CICSSystemManagement/CICSPlatform/" + cicsPlex + ". Reason: " + str(e))
    maestro.role_status = "ERROR"
    raise e

#Check if CICS Application is installed and ENABLED
try:
    request = urllib2.Request('http://'+cmciUrl+':'+cmciPort+'/CICSSystemManagement/CICSApplication/'+cicsPlex)
    request.add_header('Authorization', authheader)
    u = urllib2.urlopen(request)
    applicationData = u.read()
    logger.debug("Application data response from CICS: " + applicationData)
    
    applicationDoc = minidom.parseString(applicationData)
    applicationList = applicationDoc.getElementsByTagName('cicsapplication')
    applicationFound = False
    for applicationResource in applicationList:
        applicationName = str(applicationResource.getAttribute('application'))
        logger.debug("Found application: " + applicationName)
        if applicationName == application:
            #Found application with same name. Check in correct platform
            logger.debug("Check platform definition for application is: " + platformDef)
            if platformDef == str(applicationResource.getAttribute('platdef')):
                #Application found in same platform. Check Application version
                logger.debug("Check application version is: " + applicationVer)
                if applicationVer == str(applicationResource.getAttribute('majorversion')) + '.' + str(applicationResource.getAttribute('minorversion')) + '.' + str(applicationResource.getAttribute('microversion')):
                    #Correct version of application found in platform. Sanity check binding ID and version
                    applicationBinddir = str(applicationResource.getAttribute('binddir'))
                    if platformHome.endswith('/'):
                        requiredBinddir = platformHome + 'bindings/' + binding + '_' + bindingVer
                    else:
                        requiredBinddir = platformHome + '/bindings/' + binding + '_' + bindingVer
                    #endif                  
                    logger.debug("Check binding directory is: " + requiredBinddir + '/')
                    if applicationBinddir == (requiredBinddir + '/') or applicationBinddir == requiredBinddir:
                        #Finally check application status
                        applicationStatus = str(applicationResource.getAttribute('enablestatus'))
                        logger.debug("CICS application status is: " + applicationStatus)
                        if not applicationStatus == 'ENABLED':
                            logger.error("Existing CICS Application " + application + " is not enabled. Application " + application + " v" + applicationVer + " in platform " + platform + " in CICSplex " + cicsPlex + " has status " + applicationStatus)
                            maestro.role_status = "ERROR"
                            raise Exception("Status of existing CICS application is not ENABLED")
                        #endif
                        applicationFound = True
                        break
                    else:
                        logger.error("Installed CICS application " + application + " v" + applicationVer + " deployed with different binding ID or binding version :" + applicationBinddir)
                        maestro.role_status = "ERROR"
                        raise Exception("Installed CICS application has been deployed with a different CICS binding")
                    #endif
                else:
                    logger.debug("Incorrect version of the application has been found in the platform. The installed version is v" + str(applicationResource.getAttribute('majorversion')) + '.' + str(applicationResource.getAttribute('minorversion')) + '.' + str(applicationResource.getAttribute('microversion')) + ", not v" + applicationVer)
                #endif
            else:
                logger.debug("Application with correct name found but in different platform")
            #endif
        #endif
    #endfor
    if not applicationFound:
        logger.error("Application " + application + " version " + applicationVer + " not found in platform " + platform + " in CICSplex " + cicsPlex)
        maestro.role_status = "ERROR"
        raise Exception("Application " + application + " version " + applicationVer + " not found in platform " + platform + " in CICSplex " + cicsPlex)
    #endif
except urllib2.URLError as e:
    logger.error("Unable to receive CICS application data via CMCI http://" + cmciUrl + ":" + cmciPort + "/CICSSystemManagement/CICSApplication/" + cicsPlex + ". Reason: " + str(e))
    maestro.role_status = "ERROR"
    raise e
    
nodeName = maestro.node['name']
roleName = maestro.node['name'] + '.' + maestro.role['name']
metafile = utilpath + '/monitoring/metadata/CICSMetrics.metrics'
script = utilpath + '/monitoring/getmetrics.py'
os.chmod(script, 0775)
arguments = cmciUrl+' '+cmciPort+' '+cicsPlex+' '+base64string+' '+application+' '+applicationVer+' '+platform+' '+binding+' '+bindingVer
json_object = ('{"node":"' + nodeName + '",' + 
                '"role":"' + roleName + '",' +
                '"collector":"com.ibm.maestro.monitor.collector.script",'+
                '"config":{'+
                   '"metafile":"'+metafile+'",'+
                   '"executable":"'+script+'",'+
                   '"arguments":"'+arguments+'",'+
                   '"timeout":10'+
                '}}')
maestro.monitorAgent.register(json_object)

logger.debug("End of CICSAPP install.py")
